@@ -20,8 +20,8 @@ import keyring
 import lz4.block
 from Cryptodome.Cipher import AES
 from Cryptodome.Protocol.KDF import PBKDF2
-from pydantic import BaseModel
-from pydantic.dataclasses import dataclass
+from pydantic.v1 import BaseModel
+from pydantic.v1.dataclasses import dataclass
 from requests.auth import HTTPBasicAuth
 
 from bigeye_sdk.functions.aws import get_secret
@@ -52,7 +52,7 @@ class ApiAuth(YamlSerializable):
         pass
 
     @classmethod
-    def load(cls, auth_file: str, workspace: str = 'DEFAULT') -> ApiAuth:
+    def load(cls, auth_file: str = None, workspace: str = 'DEFAULT') -> ApiAuth:
         auth_file = cls.find_user_credentials(auth_file)
         if os.path.isfile(auth_file):
             if auth_file.lower().endswith('.json'):
