@@ -3,11 +3,11 @@ from __future__ import annotations
 import sys
 import textwrap
 from abc import ABC, abstractmethod
-from typing import List, Dict, TypeVar, Any
+from typing import List, Dict, TypeVar, Any, Union
 
 import yaml
 from betterproto import Casing
-from pydantic import Field
+from pydantic.v1 import Field
 from pydantic_yaml import YamlStrEnum
 
 from bigeye_sdk import DatawatchObject
@@ -70,6 +70,11 @@ Number of Errors: {err_cnt}
 
 {fixme_file_message}"""
 
+RECOMMEND_DISABLE = """The advanced setting 'Run metric on save' is enabled. 
+It is recommended to disable this setting when deploying metrics via bigconfig.
+Someone from Bigeye can disable it via the UI or it can disabled by an Admin via the API using the key metric.save.run_immediately
+See https://docs.bigeye.com/reference/updateconfigs"""
+
 BIGCONFIG_REPORT = TypeVar('BIGCONFIG_REPORT', bound='BigConfigReport')
 REPORTS: List[BIGCONFIG_REPORT] = []
 
@@ -84,11 +89,17 @@ def metric_type_not_supported(invalid: InvalidAssetMetricDefinitionApplication)
     ]
 
 
-def process_reports(output_path: str, strict_mode: bool, process_stage: ProcessStage):
+def process_reports(output_path: str, strict_mode: bool, process_stage: ProcessStage, run_metric_on_save: Union[bool, None]):
     report_files = []
     errors_reported = False
 
     source_reports = []
+    final_report = "{message}\n{additional_info}"
+    rd = ""
+
+    if run_metric_on_save:
+        rd = RECOMMEND_DISABLE
+
 
     for report in REPORTS:
         if process_stage == report.process_stage:
@@ -106,20 +117,21 @@ def process_reports(output_path: str, strict_mode: bool, process_stage: ProcessS
             )
         )
     elif errors_reported:
-        print(
-            FAILED_API_EXECUTION_MSG.format(
-                source_reports="\n".join(source_reports),
-                report_file_list="\n".join(report_files)
-            )
+        message = FAILED_API_EXECUTION_MSG.format(
+            source_reports="\n".join(source_reports),
+            report_file_list="\n".join(report_files)
         )
-        print(f"Strict mode is OFF. There are errors reported.")
+
+        info = f"Strict mode is OFF. There are errors reported.\n\n{rd}"
     else:
-        print(
-            SUCCESSFUL_API_EXECUTION_MSG.format(
-                source_reports="\n".join(source_reports),
-                report_file_list="\n".join(report_files)
-            )
+        message = SUCCESSFUL_API_EXECUTION_MSG.format(
+            source_reports="\n".join(source_reports),
+            report_file_list="\n".join(report_files)
         )
+        info = f"{rd}"
+
+    print(final_report.format(message=message, additional_info=info))
+    return final_report
 
 
 class BigConfigReport(File, ABC):
@@ -148,7 +160,8 @@ def raise_files_contain_error_exception(validation_error_cnt: int, unmatched_val
 
     if unmatched_validations_errors:
         formatted_unmatched_validations = '\n'.join(
-            ["ValidationError:\n" + umv.yaml(exclude_none=True, exclude_defaults=True, exclude_unset=True)
+            ["ValidationError:\n" + yaml.safe_dump(
+                umv.json(exclude_none=True, exclude_defaults=True, exclude_unset=True))
              for umv in unmatched_validations_errors])
         fixme_file_message = FIXME_FILE_MESSAGE_SOME_UNMATCHED.format(
             unmatched_validation_errors=formatted_unmatched_validations,
@@ -170,8 +183,8 @@ class ProcessStage(YamlStrEnum):
     PLAN = 'PLAN'
 
 
-class MetricSuiteReport(BigConfigReport, type='BIGCONFIG_REPORT'):
-    type = 'BIGCONFIG_REPORT'
+class MetricSuiteReport(BigConfigReport):
+    type: str = 'BIGCONFIG_REPORT'
     _exclude_defaults = False
     process_stage: ProcessStage
     source_name: str
@@ -214,24 +227,24 @@ Metric Application Error Count: {self.metric_application_error_count}
 Invalid Asset Identifier Count: {self.invalid_asset_identifier_count}
 Total Error Count: {self.tot_error_count()}"""
 
-# These are all available in the report and clutter the console.
-#         if self.row_creation_time_upsert_failure_count:
-#             source_report = f"""{source_report}
-#
-# Row creation Time Upsert Failures:
-# {yaml.safe_dump(self.row_creation_time_report['row_creation_time_upsert_failures'])}"""
-#
-#         if self.metric_application_errors:
-#             source_report = f"""{source_report}
-#
-# Metric Application Errors:
-# {yaml.safe_dump(self.metric_application_errors)}"""
-#
-#         if self.invalid_asset_identifier_errors:
-#             source_report = f"""{source_report}
-#
-# Invalid Asset Identifier Errors:
-# {yaml.safe_dump(self.invalid_asset_identifier_errors)}"""
+        # These are all available in the report and clutter the console.
+        #         if self.row_creation_time_upsert_failure_count:
+        #             source_report = f"""{source_report}
+        #
+        # Row creation Time Upsert Failures:
+        # {yaml.safe_dump(self.row_creation_time_report['row_creation_time_upsert_failures'])}"""
+        #
+        #         if self.metric_application_errors:
+        #             source_report = f"""{source_report}
+        #
+        # Metric Application Errors:
+        # {yaml.safe_dump(self.metric_application_errors)}"""
+        #
+        #         if self.invalid_asset_identifier_errors:
+        #             source_report = f"""{source_report}
+        #
+        # Invalid Asset Identifier Errors:
+        # {yaml.safe_dump(self.invalid_asset_identifier_errors)}"""
 
         return source_report
 
@@ -267,7 +280,6 @@ Total Error Count: {self.tot_error_count()}"""
         for mae in obj.metric_application_errors:
             metric_application_errors.append(mae.to_dict())
 
-
         invalid_asset_identifier_errors = [ice.to_dict() for ice in obj.invalid_cohort_errors]
 
         pr = MetricSuiteReport(
