@@ -6,7 +6,6 @@ from json import JSONDecodeError
 from typing import List, Optional, Dict, Tuple
 
 import requests
-from requests.auth import HTTPBasicAuth
 
 from bigeye_sdk.authentication.api_authentication import BasicAPIAuth, BrowserAPIAuth, ApiAuth
 from bigeye_sdk.bigconfig_validation.big_config_reports import MetricSuiteReport, ProcessStage
@@ -20,21 +19,50 @@ from bigeye_sdk.functions.delta_functions import infer_column_mappings, build_cc
 from bigeye_sdk.functions.metric_functions import set_default_model_type_for_threshold, is_freshness_metric
 from bigeye_sdk.functions.table_functions import get_table_column_priority_first, table_has_metric_time, \
     fully_qualified_table_to_elements
-from bigeye_sdk.generated.com.bigeye.models.generated import MetricConfiguration, MetricType, \
-    CreateComparisonTableResponse, CreateComparisonTableRequest, \
-    ComparisonTableConfiguration, IdAndDisplayName, ComparisonColumnMapping, \
-    ColumnNamePair, Table, GetDebugQueriesResponse, Schema, Source, MetricSuite, MetricSuiteResponse, \
-    MetricNamesResponse, WorkflowStatusResponse, WorkflowProcessingStatus, UserAuth, BigconfigWorkflowStatusResponse, \
-    MetricCreationState, LookbackType, TimeInterval, MetricParameter, NotificationChannel, Threshold
-from bigeye_sdk.log import get_logger
-from bigeye_sdk.model.delta_facade import SimpleDeltaConfiguration, SimpleDeltaConfigurationFile
-from bigeye_sdk.model.metric_facade import SimpleUpsertMetricRequest
-from bigeye_sdk.model.protobuf_extensions import MetricDebugQueries
-from bigeye_sdk.model.protobuf_message_facade import (
-    SimpleColumnMapping,
-    SimpleColumnPair,
-    SimpleNamedSchedule,
+from bigeye_sdk.generated.com.bigeye.models.generated import (
+    MetricConfiguration,
+    MetricType,
+    CreateComparisonTableResponse,
+    CreateComparisonTableRequest,
+    ComparisonTableConfiguration,
+    IdAndDisplayName,
+    ComparisonColumnMapping,
+    ColumnNamePair,
+    Table,
+    GetDebugQueriesResponse,
+    Schema,
+    Source,
+    MetricSuite,
+    MetricSuiteResponse,
+    MetricNamesResponse,
+    WorkflowStatusResponse,
+    WorkflowProcessingStatus,
+    UserAuth,
+    BigconfigWorkflowStatusResponse,
+    MetricCreationState,
+    LookbackType,
+    TimeInterval,
+    MetricParameter,
+    NotificationChannel,
+    Threshold,
+    CatalogAttribute,
+    SetCatalogAttributeRequest,
+    CatalogAttributeResponse,
+    GetCatalogAttributeRequest,
+    UpsertDeltaRequest,
+    Delta,
+    UpsertDeltaResponse,
+    OwnableType,
+    SetObjectOwnerRequest,
+    ObjectOwnerResponse, SendDbtCoreRunInfoResponse, SendDbtCoreRunInfoRequest, CustomRuleBulkRequest, BulkResponse,
+    CustomRulesThresholdType, MetricSchedule, CreateCustomRuleRequest, CustomRule, CustomRuleInfo,
 )
+from bigeye_sdk.log import get_logger
+from bigeye_sdk.model.delta_facade import SimpleDeltaConfiguration
+from bigeye_sdk.model.metric_facade import SimpleUpsertMetricRequest
+from bigeye_sdk.model.protobuf_enum_facade import SimpleCatalogEntityType
+from bigeye_sdk.model.protobuf_extensions import MetricDebugQueries
+
 
 log = get_logger(__file__)
 
@@ -169,7 +197,11 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         if not source_names:
             return {s.name: s for s in self.get_sources().sources}
         else:
-            return {s.name: s for s in self.get_sources().sources if s.name in source_names}
+            source_names = {s.name: s for s in self.get_sources().sources if s.name in source_names}
+            if not source_names:
+                raise Exception(f"No sources can be found for given source names. Please verify names and try again.")
+
+            return source_names
 
     def get_schemas_by_fq_name(self, fq_schema_names: List[str]) -> Dict[str, Schema]:
         """
@@ -303,103 +335,6 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         for t in tables:
             self.unset_table_metric_time(t)
 
-    def suggest_deltas(self,
-                       warehouse_name_pairs: List[Tuple[str, str]] = None,
-                       fq_schema_name_pairs: List[Tuple[str, str]] = None,
-                       warehouse_id_pairs: List[Tuple[int, int]] = None,
-                       schema_id_pairs: List[Tuple[int, int]] = [],
-                       table_id_pairs: List[Tuple[int, int]] = []) -> SimpleDeltaConfigurationFile:
-        """
-        Takes pairs of ids at the warehouse, schema, and table level and creates deltas between those ids.  Tables will
-        match by name based on fuzzy logic for warehouse and schema ids.  The deltas created will use defaults and infer
-        column mappings based on a string match.
-        :param warehouse_name_pairs: List containing pairs of warehouse names [source, target]
-        :param fq_schema_name_pairs: List containing pairs of fully qualified schema names [source, target]
-        :param fq_table_name_pairs: List containing pairs of fully qualified table names [source, target]
-        :param warehouse_id_pairs: list containing pairs of warehouse ids [source, target]
-        :param schema_id_pairs: pairs of schema ids. [source, target]
-        :param table_id_pairs: pairs of table ids. [source, target]
-        :return: List of simple delta configurations created. [source, target]
-        :return: a list of SimpleDeltaConfigurations that were suggested and created.
-        """
-
-        # TODO: If exists then pull the config
-        # TODO: Add configurable cron.
-
-        tidp_dict: Dict[str, Tuple[int, int]] = {}
-
-        if warehouse_name_pairs:
-            sources: Dict[str, Source] = self.get_sources_by_name()
-            widps: List[Tuple[int, int]] = [(sources[wnp[0]].id, sources[wnp[1]].id) for wnp in warehouse_name_pairs]
-
-            if warehouse_id_pairs:
-                warehouse_id_pairs.extend(widps)
-            else:
-                warehouse_id_pairs = widps
-
-        if fq_schema_name_pairs:
-            schema_id_pairs.extend(
-                [(self.get_schemas_by_fq_name(fq_schema_names=[snp[0]])[snp[0]].id,
-                  self.get_schemas_by_fq_name(fq_schema_names=[snp[1]])[snp[1]].id)
-                 for snp in fq_schema_name_pairs]
-            )
-
-        if warehouse_id_pairs:
-            for widp in warehouse_id_pairs:
-                st = self.get_tables(warehouse_id=[widp[0]])
-                tt = self.get_tables(warehouse_id=[widp[1]])
-
-                m = match_tables_by_name(source_tables=st.tables, target_tables=tt.tables)
-                tidp_dict.update(m)
-
-        if schema_id_pairs:
-            for sidp in schema_id_pairs:
-                st = self.get_tables(schema_id=[sidp[0]])
-                tt = self.get_tables(schema_id=[sidp[1]])
-
-                m = match_tables_by_name(source_tables=st.tables, target_tables=tt.tables)
-                tidp_dict.update(m)
-
-        if table_id_pairs:
-            st = self.get_tables(ids=[tid[0] for tid in table_id_pairs])
-            tt = self.get_tables(ids=[tid[1] for tid in table_id_pairs])
-            m = match_tables_by_name(source_tables=st.tables, target_tables=tt.tables)
-            tidp_dict.update(m)
-
-        created_deltas: List[SimpleDeltaConfiguration] = []
-
-        for delta_name, tidp in tidp_dict.items():
-            sdc = SimpleDeltaConfiguration(delta_name=delta_name, source_table_id=tidp[0], target_table_id=tidp[1])
-            # TODO: Clunky
-            r = self.create_deltas_from_simple_conf(sdcl=[sdc])[0]
-
-            # TODO: See if we can move the builder logic over to SimpleDeltaConfiguration.from_response_object
-            response_as_sdc = SimpleDeltaConfiguration(delta_id=r.id,
-                                                       delta_name=r.name,
-                                                       source_table_id=r.source_table_id,
-                                                       target_table_id=r.target_table_id,
-                                                       source_filters=r.source_filters,
-                                                       target_filters=r.target_filters
-                                                       )
-
-            response_as_sdc.delta_column_mapping = [SimpleColumnMapping.from_datawatch_object(cm) for cm in
-                                                    r.column_mappings]
-
-            # TODO: create a function to get the metrics common to all column mappings, filter from each column mapping,
-            # TODO: and add them to builder.all_column_metrics?
-
-            response_as_sdc.group_bys = [SimpleColumnPair.from_datawatch_object(cp) for cp in r.group_bys]
-
-            # TODO: way too cluncky.  We have to make another round trip just to get the cron.
-            cron_resp = self.get_named_schedule(ids=[r.named_schedule.id])
-
-            if cron_resp.named_schedules:
-                response_as_sdc.cron_schedule = SimpleNamedSchedule.from_datawatch_object(cron_resp.named_schedules[0])
-
-            created_deltas.append(response_as_sdc)
-
-        return SimpleDeltaConfigurationFile(type='DELTA_CONFIGURATION_FILE', deltas=created_deltas)
-
     def delete_deltas_by_name(self, delta_names: List[str]):
         """
         Deletes deltas by string name.
@@ -413,8 +348,7 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         for deltaid in existing_delta_ids:
             self.delete_delta(comparison_table_id=deltaid)
 
-    def create_deltas_from_simple_conf(self, sdcl: List[SimpleDeltaConfiguration]) -> \
-            List[ComparisonTableConfiguration]:
+    def create_deltas_from_simple_conf(self, sdcl: List[SimpleDeltaConfiguration]) -> List[Delta]:
         """
         Creates Deltas from a SimpleDeltaConfigurationList.
 
@@ -435,82 +369,145 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             source_schema: str
             target_schema: str
             source_tbl: str = ""
-            target_tbl: str = ""
             source_table = Table()
-            target_table = Table()
+            comparisons = []
+            warehouses = []
 
             if sdc.fq_source_table_name:
                 swh, source_schema, source_tbl = fully_qualified_table_to_elements(sdc.fq_source_table_name)
                 schemas.append(source_schema)
+                warehouses.append(swh)
                 table_names.append(source_tbl)
 
-            if sdc.fq_target_table_name:
-                twh, target_schema, target_tbl = fully_qualified_table_to_elements(sdc.fq_target_table_name)
-                schemas.append(target_schema)
-                table_names.append(target_tbl)
-
             if sdc.source_table_id:
                 table_ids.append(sdc.source_table_id)
 
-            if sdc.target_table_id:
-                table_ids.append(sdc.target_table_id)
+            targets = sdc.target_table_comparisons
 
-            tables = self.get_tables(ids=table_ids, schema=schemas, table_name=table_names).tables
+            target_comparisons = {}
+            target_ids = []
+            target_names = []
+            fq_target_names = []
 
-            if len(tables) == 2:
+            for t in targets:
+
+                if t.fq_target_table_name:
+                    twh, target_schema, target_tbl = fully_qualified_table_to_elements(t.fq_target_table_name)
+                    schemas.append(target_schema)
+                    target_names.append(target_tbl)
+                    warehouses.append(twh)
+                    fq_target_names.append(t.fq_target_table_name.lower())
+
+                if t.target_table_id:
+                    target_ids.append(t.target_table_id)
+
+            source_ids = [s.id for s in self.get_sources_by_name(source_names=warehouses).values()]
+
+            tables_ix = {t.id: t for t in self.get_tables(ids=table_ids+target_ids,
+                                                            schema=schemas,
+                                                            table_name=table_names+target_names,
+                                                            warehouse_id=source_ids).tables}
+
+            tables = list(tables_ix.values())
+
+            if len(tables) == 1 and len(targets) == 1:
+                t = tables[0]
+                source_table = t
+                targets[0].target_table_id = t.id
+                target_comparisons[t.id] = targets[0]
+            elif len(tables) == 1 and len(targets) == 2:
+                t = tables[0]
+                source_table = t
+                for i, target in enumerate(targets):
+                    target.target_table_id = t.id
+                    target_comparisons[f"{i}.{t.id}"] = target
+            elif len(tables) != 1:
                 for t in tables:
-                    if t.id == sdc.source_table_id or t.name == source_tbl:
+                    fq_table_name = f"{t.warehouse_name}.{t.schema.name}.{t.name}".lower()
+                    if t.id in target_ids or fq_table_name in fq_target_names:
+                        for i, target in enumerate(targets):
+                            if t.id == target.target_table_id:
+                                target_comparisons[f"{i}.{t.id}"] = target
+                            elif (f"{t.warehouse_name}.{t.schema.name}.{t.name}".lower()
+                                  == target.fq_target_table_name.lower()):
+                                target.target_table_id = t.id
+                                target_comparisons[f"{i}.{t.id}"] = target
+                    elif t.id == sdc.source_table_id or t.name == source_tbl or fq_table_name not in fq_target_names:
                         source_table = t
-                    elif t.id == sdc.target_table_id or t.name == target_tbl:
-                        target_table = t
                     else:
                         raise TableNotFoundException(f"Erroneous table id returned: {t.id}")
             else:
 
-                raise TableNotFoundException(f"Cannot find table ids. Source Table: {sdc.source_table_id}  "
-                                             f"Target Table: {sdc.target_table_id}")
+                raise TableNotFoundException(f"Cannot find table ids.")
 
-            response_as_ctc = ComparisonTableConfiguration(name=sdc.delta_name, source_table_id=source_table.id,
-                                                           target_table_id=target_table.id)
+            for target_compare in target_comparisons.values():
+                target_table = tables_ix[target_compare.target_table_id]
+                response_as_ctc = ComparisonTableConfiguration(name=sdc.delta_name, source_table_id=source_table.id,
+                                                                target_table_id=target_compare.target_table_id)
 
-            if sdc.delta_column_mapping:
-                """
-                If mappings are declared then metrics will be taken from those mappings and no defaults will be applied.
-                """
-                response_as_ctc.column_mappings = [
-                    build_ccm(scm=cm, source_table=source_table, target_table=target_table)
-                    for cm in sdc.delta_column_mapping]
-            else:
-                """
-                If no mappings are declared then column mappings will be inferred and metrics will be defaulted.
-                """
-                source_metric_types = self.get_delta_applicable_metric_types(table_id=source_table.id).metric_types
-                target_metric_types = self.get_delta_applicable_metric_types(table_id=target_table.id).metric_types
-                response_as_ctc.column_mappings = infer_column_mappings(source_metric_types=source_metric_types,
-                                                                        target_metric_types=target_metric_types)
+                if target_compare.delta_column_mapping:
+                    """
+                    If mappings are declared then metrics will be taken from those mappings and no defaults will be applied.
+                    """
+                    response_as_ctc.column_mappings = [
+                        build_ccm(scm=cm, source_table=source_table, target_table=target_table)
+                        for cm in target_compare.delta_column_mapping]
+                else:
+                    """
+                    If no mappings are declared then column mappings will be inferred and metrics will be defaulted.
+                    """
+                    source_metric_types = self.get_delta_applicable_metric_types(table_id=source_table.id).metric_types
+                    target_metric_types = self.get_delta_applicable_metric_types(table_id=target_table.id).metric_types
+                    response_as_ctc.column_mappings = infer_column_mappings(source_metric_types=source_metric_types,
+                                                                            target_metric_types=target_metric_types)
 
-            if sdc.all_column_metrics:
-                all_column_metrics = [m.to_datawatch_object() for m in sdc.all_column_metrics]
-                """If SDC has defined all_column_metrics then the each columns metrics wil lbe extended with the metrics
-                defined in all_column_metrics."""
-                for m in response_as_ctc.column_mappings:
-                    m.metrics.extend(all_column_metrics)
+                if target_compare.all_column_metrics:
+                    all_column_metrics = [m.to_datawatch_object() for m in target_compare.all_column_metrics]
+                    """If SDC has defined all_column_metrics then the each columns metrics wil lbe extended with the metrics
+                    defined in all_column_metrics."""
+                    for m in response_as_ctc.column_mappings:
+                        m.metrics.extend(all_column_metrics)
 
-            response_as_ctc.group_bys = [gb.to_datawatch_object() for gb in sdc.group_bys]
-            response_as_ctc.source_filters = sdc.source_filters
-            response_as_ctc.target_filters = sdc.target_filters
+                response_as_ctc.group_bys = [gb.to_datawatch_object() for gb in target_compare.group_bys]
+                response_as_ctc.source_filters = target_compare.source_filters
+                response_as_ctc.target_filters = target_compare.target_filters
+                response_as_ctc.tolerance = target_compare.tolerance
 
-            schedules = self.get_named_schedule().named_schedules
-            for s in schedules:
-                """Matches on cron value."""
-                if sdc.cron_schedule is None:
-                    response_as_ctc.named_schedule = IdAndDisplayName(id=s.id, display_name=s.name)
-                elif s.cron == sdc.cron_schedule.cron or s.name == sdc.cron_schedule.name:
-                    response_as_ctc.named_schedule = IdAndDisplayName(id=s.id, display_name=s.name)
+                comparisons.append(response_as_ctc)
 
-            response = self.create_delta(comparison_table_configuration=response_as_ctc)
+            named_schedule = IdAndDisplayName()
+            if sdc.cron_schedule:
+                schedules = self.get_named_schedule(search=sdc.cron_schedule.name).named_schedules
+                for s in schedules:
+                    """Matches on cron value."""
+                    if s.cron == sdc.cron_schedule.cron or s.name == sdc.cron_schedule.name:
+                        named_schedule = IdAndDisplayName(id=s.id, display_name=s.name)
+                        break
+                    else:
+                        try:
+                            schedule = self.create_named_schedule(name=sdc.cron_schedule.name, cron=sdc.cron_schedule.cron)
+                            named_schedule = IdAndDisplayName(id=schedule.id, display_name=schedule.name)
+                            break
+                        except:
+                            log.warning("Failed to create cron schedule. Verify you have access.")
+                            log.info("The delta will be created without a schedule.")
+                            break
 
-            responses.append(response.comparison_table_configuration)
+
+            delta = Delta(
+                name=sdc.delta_name,
+                source_table=IdAndDisplayName(id=source_table.id, display_name=source_table.name),
+                named_schedule=named_schedule,
+                comparison_table_configurations=comparisons,
+                notification_channels=[
+                    nc.to_datawatch_object() for nc in sdc.notification_channels
+                ]
+
+            )
+
+            response = self.upsert_delta(delta=delta)
+
+            responses.append(response.delta)
 
         return responses
 
@@ -588,6 +585,14 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         return CreateComparisonTableResponse().from_dict(response)
 
+
+    def upsert_delta(self, delta: Delta):
+
+        url = "/api/v1/deltas"
+        request = UpsertDeltaRequest(delta=delta)
+        response = self._call_datawatch(Method.POST, url, request.to_json())
+        return UpsertDeltaResponse().from_dict(response)
+
     def purge_metric_suites(self, source_names: List[str], purge_all_sources: bool = False,
                             apply: bool = False) -> List[MetricSuiteReport]:
         """
@@ -823,3 +828,138 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         return r
 
+    def get_attributes(
+            self, entity_type: SimpleCatalogEntityType, entity_id: int
+    ) -> CatalogAttributeResponse:
+        url = "/api/v1/catalog-attributes/get"
+        request = GetCatalogAttributeRequest(
+            entity_type=entity_type.to_datawatch_object(), entity_id=entity_id
+        )
+        return CatalogAttributeResponse().from_dict(
+            self._call_datawatch_impl(method=Method.POST, url=url, body=request.to_json())
+        )
+
+    def set_attributes(
+            self, entity_type: SimpleCatalogEntityType, entity_id: int, attributes: dict
+    ):
+        url = "/api/v1/catalog-attributes/set"
+        attrs = [CatalogAttribute(key=k, value=v) for k, v in attributes.items()]
+        request = SetCatalogAttributeRequest(
+            entity_type=entity_type.to_datawatch_object(),
+            entity_id=entity_id,
+            attributes=attrs,
+        )
+
+        return CatalogAttributeResponse().from_dict(
+            self._call_datawatch_impl(method=Method.POST, url=url, body=request.to_json())
+        )
+
+    def set_object_owner(self, object_type: OwnableType, object_id: int, owner: int):
+        url = "/api/v1/object-owners"
+        request = SetObjectOwnerRequest(
+            ownable_type=object_type,
+            ownable_id=object_id,
+            owner=owner
+        )
+
+        return ObjectOwnerResponse().from_dict(
+            self._call_datawatch_impl(method=Method.POST, url=url, body=request.to_json())
+        )
+
+    def batch_run_metrics(self, *, metric_ids: List[int], queue: bool):
+        """Use to run a batch of metrics and submit it to the queue or not"""
+        if queue:
+            self.run_metric_batch_async(metric_ids=metric_ids)
+        else:
+            r = self.run_metric_batch(metric_ids=metric_ids)
+            log.info(r.to_json())
+
+    def send_dbt_core_job_info(self,
+                               *,
+                               project_name: Optional[str] = None,
+                               job_name: Optional[str] = None,
+                               job_run_id: Optional[str] = None,
+                               manifest_json: str = None,
+                               run_results_json: str = None,
+                               project_url: Optional[str] = None,
+                               job_url: Optional[str] = None,
+                               job_run_url: Optional[str] = None) -> SendDbtCoreRunInfoResponse:
+
+
+        url = "/api/v1/integrations/dbt/job-run"
+        request = SendDbtCoreRunInfoRequest()
+        request.project_name = project_name
+        request.job_name = job_name
+        request.job_run_id = job_run_id
+        request.manifest_json = manifest_json
+        request.run_results_json = run_results_json
+        request.project_url = project_url
+        request.job_url = job_url
+        request.job_run_url = job_run_url
+
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        return SendDbtCoreRunInfoResponse().from_dict(response)
+
+    def bulk_run_rules(self, rule_ids: List[int]) -> BulkResponse:
+        """Run a list of rules"""
+        url = '/api/v1/custom-rules/bulk'
+        request = CustomRuleBulkRequest()
+        request.is_run = True
+        request.where.ids = rule_ids
+
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        return BulkResponse().from_dict(response)
+
+    def upsert_custom_rule(self,
+                           rule_id: Optional[int] = None,
+                           warehouse_id: Optional[int] = None,
+                           name: Optional[str] = None,
+                           sql: Optional[str] = None,
+                           threshold_type: Optional[CustomRulesThresholdType] = None,
+                           upper_threshold: Optional[float] = None,
+                           lower_threshold: Optional[float] = None,
+                           collection_ids: Optional[List[int]] = None,
+                           schedule: Optional[MetricSchedule] = None,
+                           owner_id: Optional[int] = None) -> CustomRuleInfo:
+        """Upsert a custom rule."""
+        if not rule_id or rule_id == 0:
+            if not warehouse_id or not name or not sql or not threshold_type:
+                raise Exception("For new rules, the warehouse_id, name, sql, and threshold_type fields are required.")
+
+            return self.create_custom_rule(
+                warehouse_id=warehouse_id,
+                name=name,
+                sql=sql,
+                threshold_type=threshold_type,
+                upper_threshold=upper_threshold,
+                lower_threshold=lower_threshold,
+                collection_ids=collection_ids,
+                schedule=schedule,
+                owner_id=owner_id
+            )
+
+        else:
+            custom_rule = self.get_rule_by_id(rule_id=rule_id).custom_rule
+
+            if name is not None:
+                custom_rule.name = name
+            if sql is not None:
+                custom_rule.sql = sql
+            if threshold_type is not None:
+                custom_rule.threshold_type = threshold_type
+            if upper_threshold is not None:
+                custom_rule.upper_threshold = upper_threshold
+            if lower_threshold is not None:
+                custom_rule.lower_threshold = lower_threshold
+            if collection_ids is not None:
+                custom_rule.collection_ids = collection_ids
+            if schedule is not None:
+                custom_rule.metric_schedule = schedule
+            if owner_id is not None:
+                owner = [u for u in self.get_users().users if u.id == owner_id][0]
+                custom_rule.owner = owner
+
+            return self.edit_custom_rule(
+                custom_rule=custom_rule,
+                rule_id=rule_id
+            )
