@@ -2,6 +2,8 @@ from __future__ import annotations
 
 import abc
 import json
+
+from time import sleep, time
 from typing import List, Optional, Union, Dict
 
 from bigeye_sdk.client.enum import Method
@@ -40,7 +42,6 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     Table,
     ComparisonTableInfo,
     GetComparisonTableInfosResponse,
-    RunComparisonTableResponse,
     GetComparisonTableInfosRequest,
     GetIssuesRequest,
     Issue,
@@ -82,7 +83,6 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     GetMetricTemplateListRequest,
     GetMetricTemplateListResponse,
     MetricTemplateSortField,
-    UpdateVirtualTableRequest,
     SchemaChangeSortField,
     GetSchemaChangesRequest,
     GetSchemaChangesResponse,
@@ -115,13 +115,29 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     GetVirtualTableListResponse,
     IssueMessageUpdate,
     IssueConfigUpdate,
+    IssueAssignmentUpdate,
     IssueSortField,
+    WorkflowInfo,
+    WorkflowProcessingStatus,
+    RunDeltaResponse,
+    DeltaInfo,
+    Delta,
+    GetDeltaInfosResponse,
+    RelationshipType,
+    LineageEdgeV2,
+    CreateLineageEdgeV2Request,
+    GetColumnListRequest,
+    GetColumnListResponse,
+    MetricInfo, GetTableListResponse, GetTableListRequest, GetCustomRuleListResponse,
+    CreateCustomRuleRequest, CustomRule, MetricSchedule, CustomRulesThresholdType, UpdateCustomRuleRequest,
+    CustomRuleInfo, TimeInterval, TimeIntervalType
 )
 
 
 # create logger
 from bigeye_sdk.log import get_logger
-from bigeye_sdk.model.protobuf_enum_facade import SimpleFieldType, SimpleMetricTemplateParameterType
+from bigeye_sdk.model.protobuf_enum_facade import SimpleFieldType, SimpleMetricTemplateParameterType, \
+ SimpleWorkflowProcessingStatus, SimpleTableSortField, SimpleSortDirection
 
 log = get_logger(__file__)
 
@@ -138,6 +154,7 @@ class GeneratedDatawatchClient(abc.ABC):
         """Each implementation must override this."""
         pass
 
+    # TODO: Refactor all code to use search_tables below, then deprecate
     def get_tables(self,
                    *,
                    warehouse_id: List[int] = [],
@@ -152,6 +169,64 @@ class GeneratedDatawatchClient(abc.ABC):
         tables = TableList().from_dict(response)
         return tables
 
+    def search_tables(self,
+                      *,
+                      warehouse_id: List[int] = [],
+                      schema: List[str] = [],
+                      table_name: List[str] = [],
+                      ids: List[int] = [],
+                      schema_id: List[int] = [],
+                      include_favorites: bool = False,
+                      ignore_fields: bool = False,
+                      ) -> TableList:
+        url = f"/api/v1/tables{encode_url_params(locals(), remove_keys=['api_conf'])}"
+        log.info('Getting warehouse tables.')
+        log.info(url)
+        response = self._call_datawatch(Method.GET, url)
+        tables = TableList().from_dict(response)
+        return tables
+
+    # TODO: Rename to get_tables when deprecated function is deleted.
+    def get_tables_post(self,
+                        *,
+                        source_ids: List[int] = [],
+                        schema_ids: List[int] = [],
+                        table_ids: List[int] = [],
+                        page_size: int = 0,
+                        page_cursor: str = "",
+                        sort_field: SimpleTableSortField = SimpleTableSortField.UNSPECIFIED,
+                        sort_direction: SimpleSortDirection = SimpleSortDirection.UNSPECIFIED,
+                        search: str = "",
+                        ignore_fields: bool = True) -> GetTableListResponse:
+
+        url = "/api/v1/tables/fetch"
+        request = GetTableListRequest()
+        request.source_ids = source_ids
+        request.schema_ids = schema_ids
+        request.table_ids = table_ids
+        request.page_size = page_size
+        request.page_cursor = page_cursor
+        request.sort_field = sort_field.to_datawatch_object()
+        request.sort_direction = sort_direction.to_datawatch_object()
+        request.search = search
+        request.ignore_fields = ignore_fields
+
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+
+        tlist_current = GetTableListResponse().from_dict(response)
+        tlist_total = GetTableListResponse()
+        tlist_total.tables.extend(tlist_current.tables)
+
+        while tlist_current.pagination_info.next_cursor:
+            request.page_cursor = tlist_current.pagination_info.next_cursor
+            response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+            tlist_current = GetTableListResponse().from_dict(response)
+            tlist_total.tables.extend(tlist_current.tables)
+
+        return tlist_total
+
+
+
     def get_table_ids(self,
                       warehouse_id: List[int] = [],
                       schemas: List[str] = [],
@@ -475,20 +550,64 @@ class GeneratedDatawatchClient(abc.ABC):
 
         return GetDeltaApplicableMetricTypesResponse().from_dict(response)
 
-    def run_a_delta(self, *, delta_id: int) -> ComparisonTableInfo:
+    def run_a_delta(self, *, delta_id: int, await_results: bool = False) -> Optional[Delta]:
         """
-
         Args:
             delta_id: Required.  The ID of the delta to run
+            await_results: bool. Whether to wait for delta run to complete or not, default False.
 
-        Returns:  ComparisonTableInfo
-
+        Returns:  Optional[Delta]
         """
+    
+        url = f"/api/v1/deltas/run/{delta_id}"
+        response = RunDeltaResponse().from_dict(
+            self._call_datawatch(Method.GET, url)
+        )
 
-        url = f"/api/v1/metrics/comparisons/tables/run/{delta_id}"
+        if not await_results:
+            log.info(f"Delta run has been queued for {response.delta_info.delta.name}")
+            return None
+
+        workflow_response = self.get_workflow_status(
+            workflow_id=response.workflow_info.workflow_id
+        )
+    
+        while (
+            workflow_response.status
+            == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED
+            or workflow_response.status
+            == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS
+        ):
+            log.info("Delta run in queue...")
+            sleep(10)
+            workflow_response = self.get_workflow_status(workflow_id=workflow_response.workflow_id)
+    
+        status = SimpleWorkflowProcessingStatus.from_datawatch_object(
+            workflow_response.status
+        )
+        log.info(f"Delta run completed for delta ID: {delta_id} with status {status}")
+
+        delta = self.get_delta_by_id(delta_id=delta_id)
+
+        # TODO - bug with backend that reports completed status before delta is actually complete, once fixed this can
+        #  be removed.
+        seconds_since_run = time() - delta.last_run_at_epoch_seconds
+        while seconds_since_run > 60:
+            log.info(f"Reading delta results...")
+            sleep(5)
+            delta = self.get_delta_by_id(delta_id=delta_id)
+            seconds_since_run = time() - delta.last_run_at_epoch_seconds
+
+        return delta
+
+    def get_delta_info(self, delta_id: int) -> DeltaInfo:
+        url = f"/api/v1/deltas/{delta_id}"
         response = self._call_datawatch(Method.GET, url)
 
-        return RunComparisonTableResponse().from_dict(response).comparison_table_info
+        return DeltaInfo().from_dict(response)
+
+    def get_delta_by_id(self, delta_id: int) -> Delta:
+        return self.get_delta_info(delta_id=delta_id).delta
 
     def get_delta_information(self, *, delta_ids: List[int],
                               exclude_comparison_metrics: bool = False) -> List[ComparisonTableInfo]:
@@ -508,6 +627,35 @@ class GeneratedDatawatchClient(abc.ABC):
 
         return GetComparisonTableInfosResponse().from_dict(response).comparison_table_infos
 
+    def get_deltas(self, *, comparison_table_ids: List[int] = None,
+                            exclude_comparison_metrics: bool = True) -> List[DeltaInfo]:
+        """
+            Args:
+                comparison_table_ids: Optional.  The id or IDs of tables used in deltas.
+                exclude_comparison_metrics: Optional. Whether to exclude the list of ComparisonMetricInfos
+
+            Returns:  List[ComparisonTableInfo]
+        """
+
+        url = "/api/v1/deltas/fetch"
+        request = GetComparisonTableInfosRequest()
+        request.comparison_table_ids = comparison_table_ids
+        request.exclude_comparison_metrics = exclude_comparison_metrics
+
+        response = self._call_datawatch(Method.POST, url, request.to_json())
+
+        dil_current = GetDeltaInfosResponse().from_dict(response)
+        dil_return: List[DeltaInfo] = []
+        dil_return.extend(dil_current.delta_infos)
+
+        while dil_current.pagination_info.next_cursor:
+            request.page_cursor = dil_current.pagination_info.next_cursor
+            response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+            dil_current = GetDeltaInfosResponse().from_dict(response)
+            dil_return.extend(dil_current.delta_infos)
+
+        return dil_return
+
     def get_delta_groups_information(self, *, comparison_metric_id: int) -> List[ComparisonMetricGroup]:
         """
 
@@ -538,6 +686,33 @@ class GeneratedDatawatchClient(abc.ABC):
 
         return BatchRunMetricsResponse().from_dict(response)
 
+    def run_metric_batch_async(
+            self, *, metric_ids: List[int] =[]
+    ) -> List[MetricInfo]:
+        """Batch run metrics via queue"""
+        url = "/api/v1/metrics/run/batch/queue"
+
+        batch_request = BatchRunMetricsRequest(metric_ids=metric_ids)
+
+        response = WorkflowInfo().from_dict(
+            self._call_datawatch(Method.POST, url, batch_request.to_json())
+        )
+
+        workflow_response = self.get_workflow_status(workflow_id=response.workflow_id)
+
+        while (workflow_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED or
+                workflow_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS):
+            log.info("Batch metric run in queue...")
+            sleep(10)
+            workflow_response = self.get_workflow_status(workflow_id=response.workflow_id)
+
+        status = SimpleWorkflowProcessingStatus.from_datawatch_object(workflow_response.status)
+        log.info(f"Batch metric run completed for metric IDs: {metric_ids} with status {status}")
+
+        # TODO remove this call when backend returns BatchMetricRunResponse in workflow status
+        # https://linear.app/torodata/issue/HELP-721/return-batchrunmetricsresponse-with-workflow-response
+        return self.get_metric_info_batch_post(metric_ids=metric_ids).metrics
+
     def get_sources(self) -> GetSourceListResponse:
         """Get sources"""
         url = "/api/v1/sources/fetch"
@@ -681,7 +856,11 @@ class GeneratedDatawatchClient(abc.ABC):
         return issues
 
     def get_issue(self, issue_id: int) -> Issue:
-        return self.get_issues(issue_ids=[issue_id])[0]
+        url = f'/api/v1/issues/{issue_id}'
+
+        return GetIssuesResponse().from_dict(
+            self._call_datawatch(method=Method.GET, url=url)
+        ).issue[0]
 
     def get_issue_by_key(self, issue_key: str) -> Issue:
         url = f'/api/v1/issues/company-issue/{issue_key}'
@@ -721,6 +900,13 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.PUT, url, request.to_json())
         return UpdateIssueResponse().from_dict(response).issue
 
+    def assign_issue_to_user(self, issue_id: int, user_id: int) -> Issue:
+        url = f'/api/v1/issues/{issue_id}'
+        assignment_update = IssueAssignmentUpdate(assignee=User(id=user_id))
+        request = UpdateIssueRequest(assignment_update=assignment_update)
+        response = self._call_datawatch(Method.PUT, url, request.to_json())
+        return UpdateIssueResponse().from_dict(response).issue
+
     def update_issue_summary(self, issue_id: int, summary: str, append: bool = False, is_link: bool = False):
         url = f'/api/v1/issues/{issue_id}'
         summary_text = summary
@@ -743,15 +929,14 @@ class GeneratedDatawatchClient(abc.ABC):
         return GetPreviewResponse().from_dict(self._call_datawatch(Method.GET, url))
 
     def create_named_schedule(
-            self, *, name: str = "", cron: str = ""
+            self, *, id: int = 0, name: str = "", cron: str = ""
     ) -> NamedSchedule:
-        """Create a named schedule"""
-
-        # post: "/api/v1/schedules"
+        """Upsert a named schedule. Pass schedule ID to update."""
 
         url = "/api/v1/schedules"
 
         request = CreateNamedScheduleRequest()
+        request.id = id
         request.name = name
         request.cron = cron
 
@@ -851,6 +1036,30 @@ class GeneratedDatawatchClient(abc.ABC):
         log.info(f"Deleting lineage relationship with ID: {relationship_id}")
         self._call_datawatch(Method.DELETE, url=url)
 
+    def create_lineage_edge(self,
+                            upstream_data_node_id: int,
+                            downstream_data_node_id: int,
+                            relationship_type: RelationshipType = RelationshipType.RELATIONSHIP_TYPE_LINEAGE) -> LineageEdgeV2:
+        """Create lineage relationship between two objects"""
+
+        request = CreateLineageEdgeV2Request()
+        request.relationship_type = relationship_type
+        request.upstream_data_node_id = upstream_data_node_id
+        request.downstream_data_node_id = downstream_data_node_id
+
+        url = '/api/v2/lineage/edges'
+
+        # If edge already exists, the call fails - catch and warn on this error instead
+        try:
+            response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+            return LineageEdgeV2().from_dict(response)
+        except Exception as e:
+            exists_err = f"Edge from ({request.upstream_data_node_id}, {request.downstream_data_node_id}) already exists"
+            if exists_err not in str(e):
+                raise e
+            else:
+                log.info(exists_err)
+
     def delete_lineage_relationship_for_node(self,
                                              *,
                                              data_node_id: int):
@@ -859,6 +1068,30 @@ class GeneratedDatawatchClient(abc.ABC):
         log.info(f"Deleting lineage relationships for data node {data_node_id}")
         self._call_datawatch(Method.DELETE, url=url)
 
+    def execute_lineage_workflow(self, source_id: int):
+        """Run lineage collection process for a source. For non-cloud warehouse vendors (Snowflake, Bigquery, Redshift)
+        this will only create the necessary containment edges (source -> table -> column)"""
+        url = f"/api/v1/lineage/sources/{source_id}/lineage/workflow"
+        log.info(f"Kicking off lineage workflow for source: {source_id}")
+        response: WorkflowStatusResponse = WorkflowStatusResponse().from_dict(
+            self._call_datawatch(Method.POST, url=url)
+        )
+
+        status_url = f"/api/v1/lineage/status/{response.workflow_id}"
+        while response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED or \
+                response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS:
+            log.info(f"Processing source {source_id} for lineage...")
+            time.sleep(10)
+            wsr: WorkflowStatusResponse = WorkflowStatusResponse().from_dict(
+                self._call_datawatch(Method.GET, url=status_url))
+            response.status = wsr.status
+
+        log.info(f"Workflow finished with status {response.status}")
+
+        if response.status != WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_COMPLETED:
+            err = f"Workflow was not completed. Final status: {response.status.name}"
+            raise Exception(err)
+
     def upsert_metric_template(self,
                                *,
                                id: int,
@@ -1084,13 +1317,13 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.GET, url=url)
         return WorkspaceListResponse().from_dict(response)
 
-    def create_workspace(self, workspace_name: str) -> CreateOrUpdateWorkspaceRequest:
+    def create_workspace(self, workspace_name: str) -> Workspace:
         url = '/api/v1/workspaces'
         request = CreateOrUpdateWorkspaceRequest(name=workspace_name)
         response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
         return Workspace().from_dict(response)
 
-    def update_workspace(self, workspace_name: str, workspace_id: int) -> CreateOrUpdateWorkspaceRequest:
+    def update_workspace(self, workspace_name: str, workspace_id: int) -> Workspace:
         url = f'/api/v1/workspaces/{workspace_id}'
         request = CreateOrUpdateWorkspaceRequest(name=workspace_name)
         response = self._call_datawatch(Method.PUT, url=url, body=request.to_json())
@@ -1128,13 +1361,13 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.GET, url=url)
         return GroupListResponse().from_dict(response)
 
-    def create_group(self, group_name: str) -> CreateOrUpdateGroupRequest:
+    def create_group(self, group_name: str) -> Group:
         url = '/api/v1/groups'
         request = CreateOrUpdateGroupRequest(name=group_name)
         response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
         return Group().from_dict(response)
 
-    def update_group(self, group_name: str, group_id: int) -> CreateOrUpdateGroupRequest:
+    def update_group(self, group_name: str, group_id: int) -> Group:
         url = f'/api/v1/groups/{group_id}'
         request = CreateOrUpdateGroupRequest(name=group_name)
         response = self._call_datawatch(Method.PUT, url=url, body=request.to_json())
@@ -1157,3 +1390,101 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.PUT, url=url, body=request.to_json())
         return BulkResponse().from_dict(response)
 
+    def get_columns(self, table_ids: List[int] = [], column_ids: List[int] = []) -> GetColumnListResponse:
+        url = '/api/v1/columns/fetch'
+        request = GetColumnListRequest(table_ids=table_ids, column_ids=column_ids)
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        return GetColumnListResponse().from_dict(response)
+
+    def create_custom_rule(self,
+                           warehouse_id: int,
+                           name: str,
+                           sql: str,
+                           threshold_type: CustomRulesThresholdType = None,
+                           upper_threshold: Optional[float] = None,
+                           lower_threshold: Optional[float] = None,
+                           collection_ids: Optional[List[int]] = None,
+                           schedule: Optional[MetricSchedule] = None,
+                           owner_id: Optional[int] = None) -> CustomRuleInfo:
+        """Create a new custom rule."""
+        url = '/api/v1/custom-rules'
+        rule = CustomRule()
+        rule.warehouse_id = warehouse_id
+        rule.name = name
+        rule.sql = sql
+        rule.threshold_type = threshold_type
+        rule.upper_threshold = upper_threshold
+        rule.lower_threshold = lower_threshold
+        rule.collection_ids = collection_ids or []
+
+        if not schedule:
+            rule.metric_schedule = MetricSchedule(
+                schedule_frequency=TimeInterval(
+                    interval_type=TimeIntervalType.HOURS_TIME_INTERVAL_TYPE,
+                    interval_value=24
+                )
+            )
+        else:
+            rule.metric_schedule = schedule
+
+        if owner_id:
+            owner = [u for u in self.get_users().users if u.id == owner_id][0]
+            rule.owner = owner
+
+        request = CreateCustomRuleRequest()
+        request.custom_rule = rule
+
+        # this is done so that default 0 values are not removed when converting to json
+        request_dict = request.to_dict()
+        request_dict["customRule"]["upperThreshold"] = upper_threshold
+        request_dict["customRule"]["lowerThreshold"] = lower_threshold
+        request_json = json.dumps(request_dict)
+
+        response = self._call_datawatch(Method.POST, url=url, body=request_json)
+        return CustomRuleInfo().from_dict(response)
+
+    def edit_custom_rule(self,
+                         custom_rule: CustomRule,
+                         rule_id: int) -> CustomRuleInfo:
+        """Edit an existing custom rule."""
+        url = f'/api/v1/custom-rules/{rule_id}'
+        request = UpdateCustomRuleRequest()
+        request.custom_rule = custom_rule
+        request.id = rule_id
+
+        # this is done so that default 0 values are not removed when converting to json
+        request_dict = request.to_dict()
+        request_dict["customRule"]["upperThreshold"] = request.custom_rule.upper_threshold
+        request_dict["customRule"]["lowerThreshold"] = request.custom_rule.lower_threshold
+        request_json = json.dumps(request_dict)
+
+        response = self._call_datawatch(Method.PUT, url=url, body=request_json)
+        return CustomRuleInfo().from_dict(response)
+
+    def get_rule_by_id(self, rule_id: int) -> CustomRuleInfo:
+        """Get a single rule by ID."""
+        url = f'/api/v1/custom-rules/{rule_id}'
+        response = self._call_datawatch(Method.GET, url=url)
+        return CustomRuleInfo().from_dict(response)
+
+    def get_rules(self, warehouse_id: int) -> GetCustomRuleListResponse:
+        """Get all rules for a given warehouse"""
+        url = f'/api/v1/custom-rules/warehouse/{warehouse_id}'
+        response = self._call_datawatch(Method.GET, url=url)
+
+        rlist_current = GetCustomRuleListResponse().from_dict(response)
+        rlist_total = GetCustomRuleListResponse()
+        rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        while rlist_current.pagination_info.next_cursor:
+            response.page_cursor = rlist_current.pagination_info.next_cursor
+            response = self._call_datawatch(Method.POST, url=url, body=response.to_json())
+            rlist_current = GetCustomRuleListResponse().from_dict(response)
+            rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        return rlist_total
+
+    def delete_custom_rule(self, id: int):
+        """Delete a custom rule."""
+        url = f'/api/v1/custom-rules/{id}'
+        return self._call_datawatch(Method.DELETE, url=url)
