@@ -55,4 +55,14 @@ class BigconfigIncompleteException(Exception):
 
 class WorkspaceNotSetException(Exception):
     def __init__(self, message: str):
-        self.message = message
\ No newline at end of file
+        self.message = message
+
+
+class DeltaUnhealthyException(Exception):
+    def __init__(self, message: str):
+        self.message = message
+
+
+class MetricUnhealthyException(Exception):
+    def __init__(self, message: str):
+        self.message = message
