@@ -5,12 +5,34 @@ import logging
 from typing import List, Union
 
 from bigeye_sdk.class_ext.enum_ext import EnumExtension
-from bigeye_sdk.generated.com.bigeye.models.generated import Threshold, NotificationChannel, MetricRunFailureReason, \
-    ForecastModelType, MetricInfo, MetricConfiguration, MetricParameter, MetricType, Table, LookbackType, \
-    ConstantThreshold, SimpleBound, SimpleBoundType, SlackChannelInfo
+from bigeye_sdk.generated.com.bigeye.models.generated import (
+    Threshold,
+    NotificationChannel,
+    MetricRunFailureReason,
+    ForecastModelType,
+    MetricInfo,
+    MetricConfiguration,
+    MetricParameter,
+    MetricType,
+    Table,
+    LookbackType,
+    ConstantThreshold,
+    SimpleBound,
+    SimpleBoundType,
+    SlackChannelInfo,
+    MetricRunStatus,
+)
 from bigeye_sdk.log import get_logger
-from bigeye_sdk.model.protobuf_enum_facade import SimpleMetricCategory, SimplePredefinedMetricName
-from bigeye_sdk.model.protobuf_message_facade import SimplePredefinedMetric, SimpleMetricType
+from bigeye_sdk.model.protobuf_enum_facade import (
+    SimpleMetricCategory,
+    SimplePredefinedMetricName,
+    MetricStatus,
+)
+from bigeye_sdk.model.protobuf_message_facade import (
+    SimplePredefinedMetric,
+    SimpleMetricType,
+    SimpleThreshold,
+)
 
 log = get_logger(__file__)
 
@@ -352,7 +374,7 @@ def get_time_interval_for_delay_string(delay_at_update: str, metric_type: str, u
     >>> get_time_interval_for_delay_string("1 day", "HOURS_SINCE_MAX_DATE", "10 14 * * 1")
     {'intervalValue': 38, 'intervalType': 'HOURS_TIME_INTERVAL_TYPE'}
     """
-    split_input = delay_at_update.split(" ")
+    split_input = delay_at_update.split(" ",1)
     interval_value = int(split_input[0])
     interval_type = get_proto_interval_type(split_input[1])
     if metric_type == "HOURS_SINCE_MAX_DATE":
@@ -569,6 +591,44 @@ def notification_channels_to_simple_txt(notification_channels: List[Notification
     pass
 
 
+def get_metric_status(metric_info: MetricInfo) -> MetricStatus:
+    status = metric_info.status.name
+
+    healthy_statuses = [
+        MetricRunStatus.METRIC_RUN_STATUS_OK,
+        MetricRunStatus.METRIC_RUN_STATUS_MUTABLE_OK
+    ]
+    alerting_statuses = [
+        MetricRunStatus.METRIC_RUN_STATUS_LOWERBOUND_CRITICAL,
+        MetricRunStatus.METRIC_RUN_STATUS_UPPERBOUND_CRITICAL,
+        MetricRunStatus.METRIC_RUN_STATUS_GROUPS_CRITICAL,
+        MetricRunStatus.METRIC_RUN_STATUS_MUTABLE_LOWERBOUND_CRITICAL
+    ]
+    failed_statuses = [
+        MetricRunStatus.METRIC_RUN_STATUS_GROUPS_LIMIT_FAILED
+    ]
+
+    if status in healthy_statuses:
+        return MetricStatus.HEALTHY
+    elif status in alerting_statuses:
+        return MetricStatus.ALERTING
+    elif status in failed_statuses:
+        return MetricStatus.FAILED
+    else:
+        return MetricStatus.UNKNOWN
+
+
+def get_metric_lookback_window(metric_config: MetricConfiguration) -> str:
+    if metric_config.lookback_type == LookbackType.METRIC_TIME_LOOKBACK_TYPE:
+        return f"Data Time Window ({metric_config.lookback.interval_value} Days)"
+    elif metric_config.lookback_type == LookbackType.DATA_TIME_LOOKBACK_TYPE:
+        return f"Data Time ({metric_config.lookback.interval_value} Days)"
+    elif metric_config.lookback_type == LookbackType.CLOCK_TIME_LOOKBACK_TYPE:
+        return f"Clock Time ({metric_config.lookback.interval_value} Days)"
+    elif metric_config.lookback_type == LookbackType.UNDEFINED_LOOKBACK_TYPE:
+        return "Full Scan"
+
+
 if __name__ == "__main__":
     import doctest
 
