@@ -5,7 +5,7 @@ import re
 
 import yaml
 from typing import List, Optional, Dict, Tuple, Any, Union
-from pydantic import Field, PrivateAttr, validator, root_validator
+from pydantic.v1 import Field, PrivateAttr, validator, root_validator
 
 from bigeye_sdk.bigconfig_validation.validation_functions import safe_split_dict_entry_lines, must_be_list_validator
 from bigeye_sdk.bigconfig_validation.yaml_model_base import YamlModelWithValidatorContext
@@ -34,7 +34,7 @@ class ColumnSelector(YamlModelWithValidatorContext):
     exclude: Union[List[str], str] = None
     regex: str = None
 
-    @validator('name')
+    @validator('name', allow_reuse=True)
     def must_have_split_length_4_or_5(cls, v):
         if v and (v == '' or len(v) == 0 or len(explode_fq_name(v)) not in [4, 5]):
             error_message = FQ_COL_NOT_RESOLVES_TO_COLUMN_ERRMSG.format(fq_column_name=v)
