@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import json
 import re
 from abc import ABC
 from typing import List, Optional, TypeVar, Union
@@ -7,7 +8,7 @@ from typing import List, Optional, TypeVar, Union
 import betterproto
 import yaml
 from deprecated import deprecated
-from pydantic import validator, Field, root_validator
+from pydantic.v1 import validator, Field, root_validator
 from pydantic_yaml import YamlStrEnum
 
 from bigeye_sdk.bigconfig_validation.validation_functions import safe_split_dict_entry_lines, must_be_list_validator
@@ -246,10 +247,10 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
 
 
 class SimplePredefinedMetric(SimpleMetricType, type='PREDEFINED'):
-    type = 'PREDEFINED'
+    type: str = 'PREDEFINED'
     predefined_metric: SimplePredefinedMetricName
 
-    @validator('predefined_metric', pre=True)
+    @validator('predefined_metric', pre=True, allow_reuse=True)
     def must_be_valid_metric(cls, v):
         try:
             return SimplePredefinedMetricName(v)
@@ -282,7 +283,7 @@ class SimplePredefinedMetric(SimpleMetricType, type='PREDEFINED'):
 
 
 class SimpleTemplateMetric(SimpleMetricType, type='TEMPLATE'):
-    type = 'TEMPLATE'
+    type: str = 'TEMPLATE'
     template_id: int
     aggregation_type: SimpleAggregationType
     template_name: Optional[str] = None
@@ -309,9 +310,9 @@ class SimpleNamedSchedule(YamlModelWithValidatorContext, DatawatchFacade):
 class SimpleColumnMapping(YamlModelWithValidatorContext, DatawatchFacade):
     source_column_name: str
     target_column_name: str
-    metrics: List[SimplePredefinedMetric]
+    metrics: List[SimplePredefinedMetric] = Field(default_factory=lambda: [])
 
-    @root_validator(pre=True)
+    @root_validator(pre=True, allow_reuse=True)
     def must_be_list(cls, values):
         must_be_list_validator(clazz=SimpleColumnMapping, attribute_name='metrics', values=values)
 
@@ -495,7 +496,7 @@ class SimpleAutoThreshold(SimpleThreshold, type='AUTO'):
     upper_bound_only: bool = False
     lower_bound_only: bool = False
 
-    @root_validator(pre=True)
+    @root_validator(pre=True, allow_reuse=True)
     def check_bounds(cls, values):
         if values.get('upper_bound_only') and values.get('lower_bound_only'):
             errlns = [
@@ -579,7 +580,7 @@ class SimpleRelativeThreshold(SimpleThreshold, type='RELATIVE'):
     lower_bound: Optional[float] = None
     reference_point: SimpleTimeInterval = None
 
-    @root_validator(pre=True)
+    @root_validator(pre=True, allow_reuse=True)
     def validate_lookback_or_reference_point_set(cls, values):
         # TODO use field aliases after upgrading pydantic version
         #  https://docs.pydantic.dev/2.3/usage/fields/#aliaspath-and-aliaschoices
@@ -616,7 +617,7 @@ class SimpleStdDevThreshold(SimpleThreshold, type='STDDEV'):
     lower_bound: Optional[float] = None
     reference_point: SimpleTimeInterval = None
 
-    @root_validator(pre=True)
+    @root_validator(pre=True, allow_reuse=True)
     def validate_lookback_or_reference_point_set(cls, values):
         # TODO use field aliases after upgrading pydantic version
         #  https://docs.pydantic.dev/2.3/usage/fields/#aliaspath-and-aliaschoices
@@ -687,7 +688,7 @@ class SimpleNotificationChannel(PydanticSubtypeSerializable, DatawatchFacade, AB
 
 
 class EmailNotificationChannel(SimpleNotificationChannel, type='EMAIL'):
-    type = 'EMAIL'
+    type: str = 'EMAIL'
     email: str
 
     def to_datawatch_object(self, **kwargs) -> NotificationChannel:
@@ -695,7 +696,7 @@ class EmailNotificationChannel(SimpleNotificationChannel, type='EMAIL'):
 
 
 class SlackNotificationChannel(SimpleNotificationChannel, type='SLACK'):
-    type = 'SLACK'
+    type: str = 'SLACK'
     slack: str  # TODO alias to channel_name?
     channel_id: str = ''
     thread_ts: str = ''
@@ -716,7 +717,7 @@ class SlackNotificationChannel(SimpleNotificationChannel, type='SLACK'):
 
 
 class WebhookNotificationChannel(SimpleNotificationChannel, type='WEBHOOK'):
-    type = 'WEBHOOK'
+    type: str = 'WEBHOOK'
     webhook: str
     webhook_header_key: Optional[str] = None
     webhook_header_value: Optional[str] = None
@@ -972,10 +973,11 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     metric_schedule: Optional[SimpleMetricSchedule] = None
     user_favorites: Optional[List[str]] = None
     rct_overrides: Optional[List[str]] = None
+    owner: Optional[str] = None
 
     @validator('threshold', always=True, pre=False)
     def set_default_threshold(cls, threshold, values):
-        metric_type = values.get('metric_type')
+        metric_type: str = values.get('metric_type')
 
         if threshold:
             """For the case that a user has specifically defined a metric"""
@@ -1004,7 +1006,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     @root_validator(pre=False)
     def set_default_schedule_frequency(cls, values):
         # TODO: This will change when only utilizing MetricSchedule as part of the MetricConfiguration
-        metric_type = values.get('metric_type')
+        metric_type: str = values.get('metric_type')
 
         if values.get('schedule_frequency'):
             """For the case that a user has specifically defined a metric"""
@@ -1047,8 +1049,10 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         if self.saved_metric_id:
             return safe_split_dict_entry_lines('saved_metric_id', self.saved_metric_id)
         else:
-            return self.yaml(exclude_unset=True, exclude_defaults=True,
-                             exclude_none=True, indent=True).splitlines()
+            data = self.json(
+                exclude_unset=True, exclude_defaults=True, exclude_none=True, indent=True
+            )
+            return yaml.safe_dump(json.loads(data)).splitlines()
 
     @validator('metric_type', pre=True)
     def must_be_dict_type(cls, metric_type):
@@ -1184,7 +1188,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
     @classmethod
     def from_datawatch_object(cls, obj: Union[MetricDefinition, MetricConfiguration]) -> SimpleMetricDefinition:
         builder = SimpleMetricDefinition()
-        builder.metric_type = SimpleMetricType.from_datawatch_object(obj.metric_type)
+        builder.metric_type: str = SimpleMetricType.from_datawatch_object(obj.metric_type)
         builder.metric_name = obj.name
         builder.description = obj.description
 
@@ -1218,6 +1222,9 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         if isinstance(obj, MetricConfiguration):
             builder.rct_overrides = obj.rct_override.split(",") if obj.rct_override else None
 
+        if isinstance(obj, MetricDefinition) and obj.owner_id:
+            builder.owner = str(obj.owner_id)
+
         return builder
 
     def to_datawatch_object(self, to_config: bool = False, **kwargs) -> Union[MetricDefinition, MetricConfiguration]:
@@ -1228,7 +1235,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         # Verifying that metric_type has been set before serializing to datawatch object.
         if self.metric_type:
-            builder.metric_type = self.metric_type.to_datawatch_object()
+            builder.metric_type: str = self.metric_type.to_datawatch_object()
         else:
             InvalidConfigurationException(
                 "Metric Type cannot be None.  Verify that Saved Metric IDs have been applied.")
@@ -1256,7 +1263,7 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
                 builder.lookback = self.lookback.lookback_window.to_datawatch_object()
 
             if self.lookback.lookback_type:
-                builder.lookback_type = self.lookback.lookback_type.to_datawatch_object()
+                builder.lookback_type: str = self.lookback.lookback_type.to_datawatch_object()
 
             builder.grain_seconds = self.lookback.bucket_size.to_seconds()
 
@@ -1272,6 +1279,8 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
 
         if self.user_favorites:
             builder.tags.extend(self.user_favorites)
+        if self.owner:
+            builder.owner_id = int(self.owner)
 
         if self.rct_overrides:
             if len(self.rct_overrides) == 1 and self.rct_overrides[0] == "FULL_SCAN":
