@@ -1,16 +1,14 @@
 from __future__ import annotations
 
-import re
 import abc
 import json
 import os
 from dataclasses import asdict
-from typing import TypeVar, List
+from typing import TypeVar
 
 import yaml
-from pydantic.dataclasses import dataclass
-from pydantic.json import pydantic_encoder
-from pydantic_yaml import YamlModelMixin
+from pydantic.v1.dataclasses import dataclass
+from pydantic.v1.json import pydantic_encoder
 
 from bigeye_sdk.bigconfig_validation.validation_context import put_bigeye_yaml_file_to_ix, process_validation_errors, \
     get_validation_error_cnt, get_file_error_match_cnt
@@ -112,7 +110,7 @@ class PydanticSubtypeSerializable(YamlModelWithValidatorContext):
 FILE = TypeVar('FILE', bound='File')
 
 
-class File(PydanticSubtypeSerializable, YamlModelMixin):
+class File(PydanticSubtypeSerializable):
     _exclude_defaults: bool = False
 
     @classmethod
@@ -152,15 +150,17 @@ class File(PydanticSubtypeSerializable, YamlModelMixin):
 
             raise FileLoadException(errmsg)
 
-    def save(self, output_path: str, default_file_name: str = None, custom_formatter=None) -> str:
+    def save(self, output_path: str, default_file_name: str = None, custom_formatter=None, exclude_defaults=False) -> str:
         if os.path.isfile(output_path):
             output_path = output_path
         elif os.path.isdir(output_path):
             output_path = f'{output_path}/{default_file_name}.yml'
 
         with open(output_path, 'w') as fout:
-            lines = self.yaml(exclude_unset=True, exclude_none=True,
-                              exclude_defaults=self._exclude_defaults, indent=True)
+            data = self.json(exclude_unset=True, exclude_none=True,
+                             exclude_defaults=exclude_defaults, indent=True)
+            lines = yaml.safe_dump(json.loads(data),
+                                   default_flow_style=False, sort_keys=False)
             if custom_formatter:
                 fout.writelines(custom_formatter(lines))
             else:
